/*
 * Copyright 2019 Confluent Inc.
 *
 * Licensed under the Confluent Community License (the "License"); you may not use
 * this file except in compliance with the License.  You may obtain a copy of the
 * License at
 *
 * http://www.confluent.io/confluent-community-license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 */

package io.confluent.kafka.secretregistry.rest;

import java.io.IOException;
import java.util.List;
import java.util.Properties;

import javax.ws.rs.core.Configurable;

import io.confluent.kafka.secretregistry.storage.UriInfo;
import io.confluent.rest.RestConfig;
import org.eclipse.jetty.util.resource.Resource;
import org.eclipse.jetty.util.resource.ResourceCollection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.confluent.kafka.secretregistry.exceptions.SecretRegistryException;
import io.confluent.kafka.secretregistry.rest.extensions.SecretRegistryResourceExtension;
import io.confluent.kafka.secretregistry.rest.resources.PathKeyResource;
import io.confluent.kafka.secretregistry.rest.resources.PathKeyVersionResource;
import io.confluent.kafka.secretregistry.rest.resources.PathResource;
import io.confluent.kafka.secretregistry.rest.resources.RootResource;
import io.confluent.kafka.secretregistry.storage.KafkaSecretRegistry;
import io.confluent.rest.Application;
import io.confluent.rest.RestConfigException;

public class SecretRegistryRestApplication extends Application<SecretRegistryConfig> {

  private static final Logger log = LoggerFactory.getLogger(SecretRegistryRestApplication.class);
  private KafkaSecretRegistry secretRegistry = null;
  private List<SecretRegistryResourceExtension> secretRegistryResourceExtensions = null;

  public SecretRegistryRestApplication(Properties props) throws RestConfigException {
    this(new SecretRegistryConfig(props));
  }

  public SecretRegistryRestApplication(SecretRegistryConfig config) {
    super(config);
  }


  protected KafkaSecretRegistry initSecretRegistry(SecretRegistryConfig config) {
    KafkaSecretRegistry kafkaSecretRegistry = null;
    try {
      kafkaSecretRegistry = new KafkaSecretRegistry(config);
      kafkaSecretRegistry.initStore();
      UriInfo uriInfo = KafkaSecretRegistry.getUriInfoForIdentity(
              config.getString(SecretRegistryConfig.HOST_NAME_CONFIG),
              config.getInt(RestConfig.PORT_CONFIG),
              config.getList(RestConfig.LISTENERS_CONFIG),
              config.interInstanceProtocol()
      );
      kafkaSecretRegistry.initRest(new SslFactory(config), uriInfo);
    } catch (SecretRegistryException e) {
      log.error("Error starting the secret registry", e);
      onShutdown();
      throw new RuntimeException(e);
    }
    return kafkaSecretRegistry;
  }

  @Override
  public void setupResources(Configurable<?> config, SecretRegistryConfig secretRegistryConfig) {
    secretRegistry = initSecretRegistry(secretRegistryConfig);
    secretRegistryResourceExtensions =
        secretRegistryConfig.getConfiguredInstances(
            secretRegistryConfig.definedResourceExtensionConfigName(),
            SecretRegistryResourceExtension.class
        );

    config.register(RootResource.class);
    config.register(new PathResource(secretRegistry));
    config.register(new PathKeyResource(secretRegistry));
    config.register(new PathKeyVersionResource(secretRegistry));

    if (secretRegistryResourceExtensions != null) {
      try {
        for (SecretRegistryResourceExtension
            secretRegistryResourceExtension : secretRegistryResourceExtensions) {
          secretRegistryResourceExtension.register(config, secretRegistryConfig, secretRegistry);
        }
      } catch (SecretRegistryException e) {
        log.error("Error starting the secret registry", e);
        throw new RuntimeException(e);
      }
    }
  }

  @Override
  protected ResourceCollection getStaticResources() {
    List<String> locations = config.getStaticLocations();
    if (locations != null && !locations.isEmpty()) {
      Resource[] resources = locations.stream()
          .map(Resource::newClassPathResource)
          .toArray(Resource[]::new);
      return new ResourceCollection(resources);
    } else {
      return super.getStaticResources();
    }
  }

  @Override
  public void onShutdown() {

    if (secretRegistry != null) {
      secretRegistry.close();
    }

    if (secretRegistryResourceExtensions != null) {
      for (SecretRegistryResourceExtension
          secretRegistryResourceExtension : secretRegistryResourceExtensions) {
        try {
          secretRegistryResourceExtension.close();
        } catch (IOException e) {
          log.error("Error closing the extension resource", e);
        }
      }
    }
  }

  // for testing purpose only
  public KafkaSecretRegistry secretRegistry() {
    return secretRegistry;
  }
}
